@@ -20,81 +20,6 @@
 
 #include "includes.h"
 
-
-uint32_t
-TDNFLoadYumRepo(
-    HySack hSack,
-    HyRepo hRepo,
-    int nFlags
-    )
-{
-    uint32_t dwError = 0;
-
-    if(!hSack || !hRepo)
-    {
-        dwError = ERROR_TDNF_INVALID_PARAMETER;
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    dwError = hy_sack_load_yum_repo(hSack, hRepo, nFlags);
-    BAIL_ON_TDNF_HAWKEY_ERROR(dwError);
-
-cleanup:
-    return dwError;
-
-error:
-    goto cleanup;
-}
-
-uint32_t
-TDNFInitSack(
-    PTDNF pTdnf,
-    HySack* phSack,
-    int nFlags
-    )
-{
-    uint32_t dwError = 0;
-    HySack hSack = NULL;
-    char* pszHawkeyCacheDir = NULL;
-
-    if(!pTdnf || !pTdnf->pConf || !phSack)
-    {
-        dwError = ERROR_TDNF_INVALID_PARAMETER;
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    pszHawkeyCacheDir = pTdnf->pConf->pszCacheDir;
-
-    hSack = hy_sack_create(pszHawkeyCacheDir,
-                           NULL,
-                           pTdnf->pArgs->pszInstallRoot,
-                           NULL,
-                           0);
-    if(!hSack)
-    {
-        dwError = ERROR_TDNF_INVALID_PARAMETER;
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    dwError = hy_sack_load_system_repo(hSack, NULL, nFlags);
-    BAIL_ON_TDNF_HAWKEY_ERROR(dwError);
-
-    *phSack = hSack;
-cleanup:
-    return dwError;
-
-error:
-    if(hSack)
-    {
-        hy_sack_free(hSack);
-    }
-    if(phSack)
-    {
-        *phSack = NULL;
-    }
-    goto cleanup;
-}
-
 uint32_t
 TDNFCloneCmdArgs(
     PTDNF_CMD_ARGS pCmdArgsIn,
@@ -106,9 +31,9 @@ TDNFCloneCmdArgs(
     PTDNF_CMD_ARGS pCmdArgs = NULL;
 
     dwError = TDNFAllocateMemory(
-                            1,
-                            sizeof(TDNF_CMD_ARGS),
-                            (void**)&pCmdArgs);
+                  1,
+                  sizeof(TDNF_CMD_ARGS),
+                  (void**)&pCmdArgs);
     BAIL_ON_TDNF_ERROR(dwError);
 
     pCmdArgs->nAllowErasing  = pCmdArgsIn->nAllowErasing;
@@ -130,22 +55,22 @@ TDNFCloneCmdArgs(
     pCmdArgs->nIPv6          = pCmdArgsIn->nIPv6;
 
     dwError = TDNFAllocateString(
-                         pCmdArgsIn->pszInstallRoot,
-                         &pCmdArgs->pszInstallRoot);
+                  pCmdArgsIn->pszInstallRoot,
+                  &pCmdArgs->pszInstallRoot);
     BAIL_ON_TDNF_ERROR(dwError);
 
     if(IsNullOrEmptyString(pCmdArgsIn->pszConfFile))
     {
         dwError = TDNFAllocateString(
-                             TDNF_CONF_FILE,
-                             &pCmdArgs->pszConfFile);
+                      TDNF_CONF_FILE,
+                      &pCmdArgs->pszConfFile);
         BAIL_ON_TDNF_ERROR(dwError);
     }
     else
     {
         dwError = TDNFAllocateString(
-                             pCmdArgsIn->pszConfFile,
-                             &pCmdArgs->pszConfFile);
+                      pCmdArgsIn->pszConfFile,
+                      &pCmdArgs->pszConfFile);
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
@@ -159,17 +84,16 @@ TDNFCloneCmdArgs(
 
     pCmdArgs->nCmdCount = pCmdArgsIn->nCmdCount;
     dwError = TDNFAllocateMemory(
-                            pCmdArgs->nCmdCount,
-                            sizeof(char*),
-                            (void**)&pCmdArgs->ppszCmds
-                            );
+                  pCmdArgs->nCmdCount,
+                  sizeof(char*),
+                  (void**)&pCmdArgs->ppszCmds);
     BAIL_ON_TDNF_ERROR(dwError);
-        
+
     for(nIndex = 0; nIndex < pCmdArgs->nCmdCount; ++nIndex)
     {
         dwError = TDNFAllocateString(
-                         pCmdArgsIn->ppszCmds[nIndex],
-                         &pCmdArgs->ppszCmds[nIndex]);
+                      pCmdArgsIn->ppszCmds[nIndex],
+                      &pCmdArgs->ppszCmds[nIndex]);
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
@@ -179,6 +103,15 @@ TDNFCloneCmdArgs(
                                    &pCmdArgs->pSetOpt);
         BAIL_ON_TDNF_ERROR(dwError);
     }
+    else /* if there are no setopt values, prime it to ensure non null */
+    {
+        dwError = AddSetOptWithValues(
+                      pCmdArgs,
+                      CMDOPT_KEYVALUE,
+                      TDNF_SETOPT_NAME_DUMMY,
+                      TDNF_SETOPT_VALUE_DUMMY);
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
 
     *ppCmdArgs = pCmdArgs;
 
@@ -227,9 +160,16 @@ TDNFCloneSetOpts(
                                      &pCmdOptCurrent->pszOptName);
         BAIL_ON_TDNF_ERROR(dwError);
 
-        dwError = TDNFAllocateString(pCmdOptIn->pszOptValue,
-                                     &pCmdOptCurrent->pszOptValue);
-        BAIL_ON_TDNF_ERROR(dwError);
+        if (pCmdOptCurrent->nType != CMDOPT_CURL_INIT_CB)
+        {
+           dwError = TDNFAllocateString(pCmdOptIn->pszOptValue,
+                                        &pCmdOptCurrent->pszOptValue);
+           BAIL_ON_TDNF_ERROR(dwError);
+        }
+        else
+        {
+           pCmdOptCurrent->pfnCurlConfigCB = pCmdOptIn->pfnCurlConfigCB;
+        }
 
         ppCmdOptCurrent = &(pCmdOptCurrent->pNext);
         pCmdOptIn = pCmdOptIn->pNext;
@@ -243,143 +183,88 @@ error:
     goto cleanup;
 }
 
-uint32_t
-TDNFRefreshRepo(
-    PTDNF pTdnf,
-    int nCleanMetadata,
-    PTDNF_REPO_DATA_INTERNAL pRepo
-    )
-{
-    uint32_t dwError = 0;
-    HyRepo hRepo = NULL;
-    int nYumFlags = HY_LOAD_FILELISTS | HY_LOAD_UPDATEINFO;
-    char* pszRepoCacheDir = NULL;
-    int nMetadataExpired = 0;
-
-    if(!pTdnf || !pTdnf->hSack || !pRepo || !pRepo->nEnabled)
-    {
-        dwError = ERROR_TDNF_INVALID_PARAMETER;
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    //Check if expired since last sync per metadata_expire
-    if(!nCleanMetadata && pRepo->lMetadataExpire >= 0)
-    {
-        dwError = TDNFAllocateStringPrintf(
-                      &pszRepoCacheDir,
-                      "%s/%s",
-                      pTdnf->pConf->pszCacheDir,
-                      pRepo->pszId);
-        BAIL_ON_TDNF_ERROR(dwError);
-
-        dwError = TDNFShouldSyncMetadata(
-                      pszRepoCacheDir,
-                      pRepo->lMetadataExpire,
-                      &nMetadataExpired);
-        BAIL_ON_TDNF_ERROR(dwError);
-
-        TDNF_SAFE_FREE_MEMORY(pszRepoCacheDir);
-        pszRepoCacheDir = NULL;
-    }
-
-    if(nCleanMetadata || nMetadataExpired)
-    {
-        if(!pTdnf->pArgs->nQuiet)
-        {
-            fprintf(stdout,
-                    "Refreshing metadata for: '%s'\n",
-                    pRepo->pszName);
-        }
-        dwError = TDNFRepoRemoveCache(pTdnf, pRepo->pszId);
-        if(dwError == ERROR_TDNF_FILE_NOT_FOUND)
-        {
-            dwError = 0;//Ignore non existent folders
-        }
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    dwError = TDNFInitRepo(pTdnf, pRepo, &hRepo);
-    BAIL_ON_TDNF_ERROR(dwError);
-
-    if(pRepo->hRepo)
-    {
-        hy_repo_free(pRepo->hRepo);
-        pRepo->hRepo = NULL;
-    }
-    pRepo->hRepo = hRepo;
-
-    dwError = TDNFLoadYumRepo(pTdnf->hSack, hRepo, nYumFlags);
-
-    if(dwError)
-    {
-        fprintf(
-            stderr,
-            "Error: Failed to synchronize cache for repo '%s' from '%s'\n",
-            pRepo->pszName, pRepo->pszBaseUrl);
-
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-
-cleanup:
-    TDNF_SAFE_FREE_MEMORY(pszRepoCacheDir);
-    return dwError;
-
-error:
-    if(pRepo)
-    {
-        pRepo->hRepo = NULL;
-        if(pRepo->nEnabled && pTdnf)
-        {
-            TDNFRepoRemoveCache(pTdnf, pRepo->pszId);
-        }
-        if(pRepo->nSkipIfUnavailable)
-        {
-            pRepo->nEnabled = 0;
-            fprintf(stdout, "Disabling Repo: '%s'\n", pRepo->pszName);
-            dwError = 0;
-        }
-    }
-    if(hRepo)
-    {
-        hy_repo_free(hRepo);
-    }
-
-    goto cleanup;
-}
-
 uint32_t
 TDNFRefreshSack(
     PTDNF pTdnf,
+    PSolvSack pSack,
     int nCleanMetadata
     )
 {
     uint32_t dwError = 0;
-
-    if(!pTdnf)
+    char* pszRepoCacheDir = NULL;
+    int nMetadataExpired = 0;
+    if(!pTdnf || !pTdnf->pArgs)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
-    if(pTdnf->hSack)
+    if (nCleanMetadata == 1)
     {
-        hy_sack_free(pTdnf->hSack);
-        pTdnf->hSack = NULL;
+        pTdnf->pArgs->nRefresh = 1;
     }
 
-    dwError = TDNFInitSack(pTdnf, &pTdnf->hSack, HY_LOAD_FILELISTS);
-    BAIL_ON_TDNF_ERROR(dwError);
-
     //If there is an empty repo directory, do nothing
     if(pTdnf->pRepos)
     {
         PTDNF_REPO_DATA_INTERNAL pTempRepo = pTdnf->pRepos;
         while(pTempRepo)
         {
+            nMetadataExpired = 0;
             if(pTempRepo->nEnabled)
             {
-                dwError = TDNFRefreshRepo(pTdnf, nCleanMetadata, pTempRepo);
+                //Check if expired since last sync per metadata_expire
+                if(pTempRepo->lMetadataExpire >= 0)
+                {
+                    dwError = TDNFAllocateStringPrintf(
+                                  &pszRepoCacheDir,
+                                  "%s/%s",
+                                  pTdnf->pConf->pszCacheDir,
+                                  pTempRepo->pszId);
+                    BAIL_ON_TDNF_ERROR(dwError);
+
+                    dwError = TDNFShouldSyncMetadata(
+                                  pszRepoCacheDir,
+                                  pTempRepo->lMetadataExpire,
+                                  &nMetadataExpired);
+                    BAIL_ON_TDNF_ERROR(dwError);
+
+                    TDNF_SAFE_FREE_MEMORY(pszRepoCacheDir);
+                    pszRepoCacheDir = NULL;
+                }
+
+                if(nMetadataExpired)
+                {
+                    dwError = TDNFRepoRemoveCache(pTdnf, pTempRepo->pszId);
+                    if(dwError == ERROR_TDNF_FILE_NOT_FOUND)
+                    {
+                        dwError = 0;//Ignore non existent folders
+                    }
+                    BAIL_ON_TDNF_ERROR(dwError);
+
+                    dwError = TDNFRemoveSolvCache(pTdnf, pTempRepo->pszId);
+                    if(dwError == ERROR_TDNF_FILE_NOT_FOUND)
+                    {
+                        dwError = 0;//Ignore non existent folders
+                    }
+                    BAIL_ON_TDNF_ERROR(dwError);
+                }
+
+                if(pSack)
+                {
+                    dwError = TDNFInitRepo(pTdnf, pTempRepo, pSack);
+                }
+                if(dwError)
+                {
+                    if(pTempRepo->nSkipIfUnavailable)
+                    {
+                        pTempRepo->nEnabled = 0;
+                        printf("Disabling Repo: '%s'\n",
+                                pTempRepo->pszName);
+
+                        dwError = 0;
+                    }
+                }
                 BAIL_ON_TDNF_ERROR(dwError);
             }
             pTempRepo = pTempRepo->pNext;
@@ -387,6 +272,7 @@ TDNFRefreshSack(
     }
 
 cleanup:
+    TDNF_SAFE_FREE_MEMORY(pszRepoCacheDir);
     return dwError;
 
 error:
