@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 VMware, Inc. All Rights Reserved.
+ * Copyright (C) 2015-2020 VMware, Inc. All Rights Reserved.
  *
  * Licensed under the GNU Lesser General Public License v2.1 (the "License");
  * you may not use this file except in compliance with the License. The terms
@@ -25,7 +25,7 @@ uint32_t
 TDNFInitRepo(
     PTDNF pTdnf,
     PTDNF_REPO_DATA_INTERNAL pRepoData,
-    HyRepo* phRepo
+    PSolvSack pSack
     )
 {
     uint32_t dwError = 0;
@@ -34,15 +34,19 @@ TDNFInitRepo(
     char* pszLastRefreshMarker = NULL;
     PTDNF_REPO_METADATA pRepoMD = NULL;
     PTDNF_CONF pConf = NULL;
-    HyRepo hRepo = NULL;
+    Repo* pRepo = NULL;
+    Pool* pPool = NULL;;
+    int nUseMetaDataCache = 0;
+    PSOLV_REPO_INFO_INTERNAL pSolvRepoInfo = NULL;
 
-    if(!pTdnf || !pTdnf->pConf || !pRepoData || !phRepo)
+    if(!pTdnf || !pTdnf->pConf || !pRepoData || !pSack || !pSack->pPool)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
     pConf = pTdnf->pConf;
+    pPool = pSack->pPool;
 
     dwError = TDNFAllocateStringPrintf(
                   &pszRepoCacheDir,
@@ -64,17 +68,38 @@ TDNFInitRepo(
                             &pRepoMD);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    //Create and set repo properties
-    hRepo = hy_repo_create(pRepoData->pszId);
-    if(!hRepo)
-    {
-        dwError = ERROR_TDNF_HAWKEY_FAILED;
-        BAIL_ON_TDNF_ERROR(dwError);
-    }
-
-    dwError = TDNFInitRepoFromMetadata(hRepo, pRepoMD);
+    dwError = TDNFAllocateMemory(
+                  1,
+                  sizeof(SOLV_REPO_INFO_INTERNAL),
+                  (void**)&pSolvRepoInfo);
     BAIL_ON_TDNF_ERROR(dwError);
 
+    pRepo = repo_create(pPool, pRepoData->pszId);
+
+    if (!pRepo)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+    pSolvRepoInfo->pRepo = pRepo;
+    pRepo->appdata = pSolvRepoInfo;
+
+    dwError = SolvCalculateCookieForRepoMD(pRepoMD->pszRepoMD, pSolvRepoInfo->cookie);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    pSolvRepoInfo->nCookieSet = 1;
+    dwError = SolvUseMetaDataCache(pSack, pSolvRepoInfo, &nUseMetaDataCache);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    if (nUseMetaDataCache == 0)
+    {
+        dwError = TDNFInitRepoFromMetadata(pSolvRepoInfo->pRepo, pRepoData->pszId, pRepoMD);
+        BAIL_ON_TDNF_ERROR(dwError);
+
+        dwError = SolvCreateMetaDataCache(pSack, pSolvRepoInfo);
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+    pool_createwhatprovides(pPool);
     dwError = TDNFAllocateStringPrintf(
                   &pszLastRefreshMarker,
                   "%s/%s",
@@ -85,17 +110,18 @@ TDNFInitRepo(
     dwError = TDNFTouchFile(pszLastRefreshMarker);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    *phRepo = hRepo;
-
 cleanup:
     TDNFFreeRepoMetadata(pRepoMD);
     TDNF_SAFE_FREE_MEMORY(pszLastRefreshMarker);
     TDNF_SAFE_FREE_MEMORY(pszRepoDataDir);
     TDNF_SAFE_FREE_MEMORY(pszRepoCacheDir);
-
+    TDNF_SAFE_FREE_MEMORY(pSolvRepoInfo);
     return dwError;
-
 error:
+    if(pRepo)
+    {
+        repo_free(pRepo, 1);
+    }
     //If there is an error during init, log the error
     //remove any cache data that could be potentially corrupt.
     if(pRepoData)
@@ -109,57 +135,39 @@ error:
         if(pTdnf)
         {
             TDNFRepoRemoveCache(pTdnf, pRepoData->pszId);
+            TDNFRemoveSolvCache(pTdnf, pRepoData->pszId);
+            TDNFRemoveLastRefreshMarker(pTdnf, pRepoData->pszId);
         }
     }
-    if(phRepo)
-    {
-        *phRepo = NULL;
-    }
-    if(hRepo)
-    {
-       hy_repo_free(hRepo);
-    }
     goto cleanup;
 }
-
 uint32_t
 TDNFInitRepoFromMetadata(
-    HyRepo hRepo,
+    Repo *pRepo,
+    const char* pszRepoName,
     PTDNF_REPO_METADATA pRepoMD
     )
 {
     uint32_t dwError = 0;
 
-    if(!hRepo || !pRepoMD)
+    if(!pRepo || !pRepoMD || IsNullOrEmptyString(pszRepoName))
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
-    hy_repo_set_string(hRepo, HY_REPO_MD_FN, pRepoMD->pszRepoMD);
-
-    if(!IsNullOrEmptyString(pRepoMD->pszPrimary))
-    {
-        hy_repo_set_string(hRepo, HY_REPO_PRIMARY_FN, pRepoMD->pszPrimary);
-    }
-
-    if(!IsNullOrEmptyString(pRepoMD->pszFileLists))
-    {
-        hy_repo_set_string(hRepo, HY_REPO_FILELISTS_FN, pRepoMD->pszFileLists);
-    }
-
-    if(!IsNullOrEmptyString(pRepoMD->pszUpdateInfo))
-    {
-        hy_repo_set_string(hRepo,
-                           HY_REPO_UPDATEINFO_FN,
-                           pRepoMD->pszUpdateInfo);
-    }
-
+    dwError = SolvReadYumRepo(pRepo,
+                  pszRepoName,
+                  pRepoMD->pszRepoMD,
+                  pRepoMD->pszPrimary,
+                  pRepoMD->pszFileLists,
+                  pRepoMD->pszUpdateInfo);
 cleanup:
     return dwError;
 
 error:
     goto cleanup;
+    return dwError;
 }
 
 uint32_t
@@ -175,7 +183,7 @@ TDNFGetGPGCheck(
     int nGPGCheck = 0;
     char* pszUrlGPGKey = NULL;
 
-    if(!pTdnf || !ppszUrlGPGKey || IsNullOrEmptyString(pszRepo))
+    if(!pTdnf || !ppszUrlGPGKey || IsNullOrEmptyString(pszRepo) || !pnGPGCheck)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
@@ -216,6 +224,145 @@ error:
     goto cleanup;
 }
 
+uint32_t
+TDNFGetSkipSignatureOption(
+    PTDNF pTdnf,
+    uint32_t *pdwSkipSignature
+    )
+{
+    uint32_t dwError = 0;
+    PTDNF_CMD_OPT pSetOpt = NULL;
+    uint32_t dwSkipSignature = 0;
+
+    if(!pTdnf || !pTdnf->pArgs)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    pSetOpt = pTdnf->pArgs->pSetOpt;
+
+    while(pSetOpt)
+    {
+        if(pSetOpt->nType == CMDOPT_KEYVALUE &&
+          !strcasecmp(pSetOpt->pszOptName, "skipsignature"))
+        {
+            dwSkipSignature = 1;
+            break;
+        }
+        pSetOpt = pSetOpt->pNext;
+    }
+    *pdwSkipSignature = dwSkipSignature;
+cleanup:
+    return dwError;
+
+error:
+    if(pdwSkipSignature)
+    {
+       *pdwSkipSignature = 0;
+    }
+    goto cleanup;
+}
+
+uint32_t
+TDNFGetSkipDigestOption(
+    PTDNF pTdnf,
+    uint32_t *pdwSkipDigest
+    )
+{
+    uint32_t dwError = 0;
+    PTDNF_CMD_OPT pSetOpt = NULL;
+    uint32_t dwSkipDigest = 0;
+
+    if(!pTdnf || !pTdnf->pArgs)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    pSetOpt = pTdnf->pArgs->pSetOpt;
+
+    while(pSetOpt)
+    {
+        if(pSetOpt->nType == CMDOPT_KEYVALUE &&
+          !strcasecmp(pSetOpt->pszOptName, "skipdigest"))
+        {
+            dwSkipDigest = 1;
+            break;
+        }
+        pSetOpt = pSetOpt->pNext;
+    }
+    *pdwSkipDigest = dwSkipDigest;
+cleanup:
+    return dwError;
+
+error:
+    if(pdwSkipDigest)
+    {
+       *pdwSkipDigest = 0;
+    }
+    goto cleanup;
+}
+
+uint32_t
+TDNFGetGPGSignatureCheck(
+    PTDNF pTdnf,
+    const char* pszRepo,
+    int* pnGPGSigCheck,
+    char** ppszUrlGPGKey
+    )
+{
+    uint32_t dwError = 0;
+    PTDNF_REPO_DATA_INTERNAL pRepo = NULL;
+    int nGPGSigCheck = 0;
+    uint32_t dwSkipSignature = 0;
+    char* pszUrlGPGKey = NULL;
+
+    if(!pTdnf || !ppszUrlGPGKey || IsNullOrEmptyString(pszRepo) || !pnGPGSigCheck)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    dwError = TDNFGetSkipSignatureOption(pTdnf, &dwSkipSignature);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    if(!(pTdnf->pArgs->nNoGPGCheck || dwSkipSignature))
+    {
+        dwError = TDNFGetRepoById(pTdnf, pszRepo, &pRepo);
+        if(dwError == ERROR_TDNF_NO_DATA)
+        {
+            dwError = 0;
+        }
+        BAIL_ON_TDNF_ERROR(dwError);
+        if(pRepo)
+        {
+            nGPGSigCheck = pRepo->nGPGCheck;
+            if(nGPGSigCheck)
+            {
+                dwError = TDNFAllocateString(
+                             pRepo->pszUrlGPGKey,
+                             &pszUrlGPGKey);
+                BAIL_ON_TDNF_ERROR(dwError);
+            }
+        }
+    }
+
+    *pnGPGSigCheck = nGPGSigCheck;
+    *ppszUrlGPGKey = pszUrlGPGKey;
+
+cleanup:
+    return dwError;
+
+error:
+    if(ppszUrlGPGKey)
+    {
+        *ppszUrlGPGKey = NULL;
+    }
+    TDNF_SAFE_FREE_MEMORY(pszUrlGPGKey);
+    goto cleanup;
+}
+
 uint32_t
 TDNFGetRepoById(
     PTDNF pTdnf,
@@ -265,6 +412,53 @@ error:
     goto cleanup;
 }
 
+uint32_t
+TDNFEventRepoMDDownloadEnd(
+    PTDNF pTdnf,
+    const char *pcszRepoId,
+    const char *pcszRepoMDUrl,
+    const char *pcszRepoMDFile
+    )
+{
+    uint32_t dwError = 0;
+    TDNF_EVENT_CONTEXT stContext = {0};
+
+    if (!pTdnf ||
+        IsNullOrEmptyString(pcszRepoId) ||
+        IsNullOrEmptyString(pcszRepoMDUrl) ||
+        IsNullOrEmptyString(pcszRepoMDFile))
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    stContext.nEvent = MAKE_PLUGIN_EVENT(
+                           TDNF_PLUGIN_EVENT_TYPE_REPO_MD,
+                           TDNF_PLUGIN_EVENT_STATE_DOWNLOAD,
+                           TDNF_PLUGIN_EVENT_PHASE_END);
+    dwError = TDNFAddEventDataString(&stContext,
+                  TDNF_EVENT_ITEM_REPO_ID,
+                  pcszRepoId);
+    BAIL_ON_TDNF_ERROR(dwError);
+    dwError = TDNFAddEventDataString(&stContext,
+                  TDNF_EVENT_ITEM_REPO_MD_URL,
+                  pcszRepoMDUrl);
+    BAIL_ON_TDNF_ERROR(dwError);
+    dwError = TDNFAddEventDataString(&stContext,
+                  TDNF_EVENT_ITEM_REPO_MD_FILE,
+                  pcszRepoMDFile);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    dwError = TDNFPluginRaiseEvent(pTdnf, &stContext);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+cleanup:
+    TDNFFreeEventData(stContext.pData);
+    return dwError;
+error:
+    goto cleanup;
+}
+
 uint32_t
 TDNFGetRepoMD(
     PTDNF pTdnf,
@@ -276,8 +470,15 @@ TDNFGetRepoMD(
     uint32_t dwError = 0;
     char *pszRepoMDFile = NULL;
     char *pszRepoMDUrl = NULL;
+    char *pszTmpRepoDataDir = NULL;
+    char *pszTmpRepoMDFile = NULL;
     PTDNF_REPO_METADATA pRepoMDRel = NULL;
     PTDNF_REPO_METADATA pRepoMD = NULL;
+    unsigned char pszCookie[SOLV_COOKIE_LEN] = {0};
+    unsigned char pszTmpCookie[SOLV_COOKIE_LEN] = {0};
+    int nNeedDownload = 0;
+    int nNewRepoMDFile = 0;
+    int nReplaceRepoMD = 0;
 
     if(!pTdnf ||
        !pRepoData ||
@@ -319,28 +520,92 @@ TDNFGetRepoMD(
     dwError = TDNFAllocateString(pRepoData->pszId, &pRepoMDRel->pszRepo);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    if(access(pszRepoMDFile, F_OK))
+    /* if repomd.xml file is not present, set flag to download */
+    if (access(pszRepoMDFile, F_OK))
     {
         if(errno != ENOENT)
         {
             dwError = errno;
             BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
         }
+        nNeedDownload = 1;
+    }
+    /* if refresh flag is set, get shasum of existing repomd file */
+    else if (pTdnf->pArgs->nRefresh)
+    {
+        dwError = SolvCalculateCookieForRepoMD(pszRepoMDFile, pszCookie);
+        BAIL_ON_TDNF_ERROR(dwError);
+        nNeedDownload = 1;
+    }
 
-        dwError = TDNFUtilsMakeDirs(pszRepoDataDir);
+    /* download repomd.xml to tmp */
+    if(nNeedDownload)
+    {
+        if(!pTdnf->pArgs->nQuiet)
+        {
+           printf("Refreshing metadata for: '%s'\n",
+                    pRepoData->pszName);
+        }
+        /* always download to tmp */
+        dwError = TDNFAllocateStringPrintf(
+                      &pszTmpRepoDataDir,
+                      "%s/tmp",
+                      pszRepoDataDir);
+        BAIL_ON_TDNF_ERROR(dwError);
+
+        dwError = TDNFUtilsMakeDirs(pszTmpRepoDataDir);
         if(dwError == ERROR_TDNF_ALREADY_EXISTS)
         {
             dwError = 0;
         }
         BAIL_ON_TDNF_ERROR(dwError);
 
+        dwError = TDNFAllocateStringPrintf(
+                      &pszTmpRepoMDFile,
+                      "%s/%s",
+                      pszTmpRepoDataDir,
+                      TDNF_REPO_METADATA_FILE_NAME);
+        BAIL_ON_TDNF_ERROR(dwError);
+
         dwError = TDNFDownloadFile(
                       pTdnf,
                       pRepoData->pszId,
                       pszRepoMDUrl,
-                      pszRepoMDFile,
+                      pszTmpRepoMDFile,
                       pRepoData->pszId);
         BAIL_ON_TDNF_ERROR(dwError);
+
+        /* plugin event indicating a repomd download happened */
+        dwError = TDNFEventRepoMDDownloadEnd(
+                      pTdnf,
+                      pRepoData->pszId,
+                      pszRepoMDUrl,
+                      pszTmpRepoMDFile);
+        BAIL_ON_TDNF_ERROR(dwError);
+
+        nNewRepoMDFile = 1;
+    }
+
+    if (nNewRepoMDFile)
+    {
+        nReplaceRepoMD = 1;
+        /* compare shasum if applicable */
+        if (pszCookie[0])
+        {
+            dwError = SolvCalculateCookieForRepoMD(pszTmpRepoMDFile, pszTmpCookie);
+            BAIL_ON_TDNF_ERROR(dwError);
+
+            if (!memcmp (pszCookie, pszTmpCookie, sizeof(pszTmpCookie)))
+            {
+                nReplaceRepoMD = 0;
+            }
+        }
+
+        if (nReplaceRepoMD)
+        {
+            dwError = TDNFReplaceRepoMDFile(pszTmpRepoMDFile, pszRepoMDFile);
+            BAIL_ON_TDNF_ERROR(dwError);
+        }
     }
 
     dwError = TDNFParseRepoMD(pRepoMDRel);
@@ -357,11 +622,15 @@ TDNFGetRepoMD(
 
 cleanup:
     TDNFFreeRepoMetadata(pRepoMDRel);
+    TDNFRemoveTmpRepodata(pszTmpRepoDataDir, pszTmpRepoMDFile);
+    TDNF_SAFE_FREE_MEMORY(pszTmpRepoMDFile);
+    TDNF_SAFE_FREE_MEMORY(pszTmpRepoDataDir);
     TDNF_SAFE_FREE_MEMORY(pszRepoMDFile);
     TDNF_SAFE_FREE_MEMORY(pszRepoMDUrl);
     return dwError;
 
 error:
+    fprintf(stderr, "Error: %s %d\n", __FUNCTION__, dwError);
     TDNFFreeRepoMetadata(pRepoMD);
     goto cleanup;
 }
@@ -595,8 +864,7 @@ TDNFParseRepoMD(
     dwError = repo_add_repomdxml(pRepo, fp, 0);
     if(dwError)
     {
-        fprintf(stdout,
-                "Error(%d) parsing repomd: %s\n",
+        printf("Error(%u) parsing repomd: %s\n",
                 dwError,
                 pool_errstr(pPool));
     }
@@ -625,6 +893,10 @@ TDNFParseRepoMD(
     BAIL_ON_TDNF_ERROR(dwError);
 
 cleanup:
+    if (fp)
+    {
+        fclose(fp);
+    }
     if(pRepo)
     {
         repo_free(pRepo, 0);
@@ -656,3 +928,25 @@ TDNFFreeRepoMetadata(
     TDNF_SAFE_FREE_MEMORY(pRepoMD->pszUpdateInfo);
     TDNF_SAFE_FREE_MEMORY(pRepoMD);
 }
+
+uint32_t
+TDNFReplaceRepoMDFile(
+    const char *pszSrcFile,
+    const char *pszDstFile
+    )
+{
+    uint32_t dwError = 0;
+
+    if (IsNullOrEmptyString(pszSrcFile) || IsNullOrEmptyString(pszDstFile))
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+    dwError = rename (pszSrcFile, pszDstFile);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+cleanup:
+    return dwError;
+error:
+    goto cleanup;
+}
