@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 VMware, Inc. All Rights Reserved.
+ * Copyright (C) 2015-2020 VMware, Inc. All Rights Reserved.
  *
  * Licensed under the GNU Lesser General Public License v2.1 (the "License");
  * you may not use this file except in compliance with the License. The terms
@@ -50,7 +50,7 @@ TDNFLoadRepoData(
     if(pDir == NULL)
     {
         dwError = ERROR_TDNF_REPO_DIR_OPEN;
-        BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+        BAIL_ON_TDNF_ERROR(dwError);
     }
 
     while ((pEnt = readdir (pDir)) != NULL )
@@ -122,6 +122,74 @@ error:
     goto cleanup;
 }
 
+uint32_t
+TDNFEventRepoReadConfigEnd(
+    PTDNF pTdnf,
+    PCONF_SECTION pSection
+    )
+{
+    uint32_t dwError = 0;
+    TDNF_EVENT_CONTEXT stContext = {0};
+
+    if (!pTdnf || !pSection)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    stContext.nEvent = MAKE_PLUGIN_EVENT(
+                           TDNF_PLUGIN_EVENT_TYPE_REPO,
+                           TDNF_PLUGIN_EVENT_STATE_READCONFIG,
+                           TDNF_PLUGIN_EVENT_PHASE_END);
+    dwError = TDNFAddEventDataPtr(&stContext,
+                  TDNF_EVENT_ITEM_REPO_SECTION,
+                  pSection);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    dwError = TDNFPluginRaiseEvent(pTdnf, &stContext);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+cleanup:
+    TDNFFreeEventData(stContext.pData);
+    return dwError;
+error:
+    goto cleanup;
+}
+
+uint32_t
+TDNFEventRepoReadConfigStart(
+    PTDNF pTdnf,
+    PCONF_SECTION pSection
+    )
+{
+    uint32_t dwError = 0;
+    TDNF_EVENT_CONTEXT stContext = {0};
+
+    if (!pTdnf || !pSection)
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    stContext.nEvent = MAKE_PLUGIN_EVENT(
+                           TDNF_PLUGIN_EVENT_TYPE_REPO,
+                           TDNF_PLUGIN_EVENT_STATE_READCONFIG,
+                           TDNF_PLUGIN_EVENT_PHASE_START);
+    dwError = TDNFAddEventDataPtr(&stContext,
+                  TDNF_EVENT_ITEM_REPO_SECTION,
+                  pSection);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    dwError = TDNFPluginRaiseEvent(pTdnf, &stContext);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+cleanup:
+    TDNFFreeEventData(stContext.pData);
+    return dwError;
+error:
+    goto cleanup;
+}
+
 uint32_t
 TDNFLoadReposFromFile(
     PTDNF pTdnf,
@@ -129,10 +197,9 @@ TDNFLoadReposFromFile(
     PTDNF_REPO_DATA_INTERNAL* ppRepos
     )
 {
+    char *pszRepo = NULL;
     uint32_t dwError = 0;
-
-    char* pszRepo = NULL;
-    char* pszMetadataExpire = NULL;
+    char *pszMetadataExpire = NULL;
 
     PTDNF_REPO_DATA_INTERNAL pRepos = NULL;
     PTDNF_REPO_DATA_INTERNAL pRepo = NULL;
@@ -142,9 +209,14 @@ TDNFLoadReposFromFile(
 
     dwError = TDNFReadConfigFile(pszRepoFile, 0, &pData);
     BAIL_ON_TDNF_ERROR(dwError);
+
     pSections = pData->pSections;
     for(; pSections; pSections = pSections->pNext)
     {
+        /* plugin event repo readconfig start */
+        dwError = TDNFEventRepoReadConfigStart(pTdnf, pSections);
+        BAIL_ON_TDNF_ERROR(dwError);
+
         pszRepo = pSections->pszName;
 
         dwError = TDNFAllocateMemory(
@@ -234,6 +306,10 @@ TDNFLoadReposFromFile(
         TDNF_SAFE_FREE_MEMORY(pszMetadataExpire);
         pszMetadataExpire = NULL;
 
+        /* plugin event repo readconfig end */
+        dwError = TDNFEventRepoReadConfigEnd(pTdnf, pSections);
+        BAIL_ON_TDNF_ERROR(dwError);
+
         pRepo->pNext = pRepos;
         pRepos = pRepo;
         pRepo = NULL;
@@ -393,9 +469,7 @@ TDNFCloneRepo(
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
-    dwError = TDNFAllocateMemory(1,
-                                 sizeof(TDNF_REPO_DATA),
-                                 (void**)&pRepo);
+    dwError = TDNFAllocateMemory(1, sizeof(TDNF_REPO_DATA), (void**)&pRepo);
     BAIL_ON_TDNF_ERROR(dwError);
 
     pRepo->nEnabled = pRepoIn->nEnabled;
@@ -409,10 +483,14 @@ TDNFCloneRepo(
     dwError = TDNFSafeAllocateString(pRepoIn->pszBaseUrl, &pRepo->pszBaseUrl);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    dwError = TDNFSafeAllocateString(pRepoIn->pszMetaLink, &pRepo->pszMetaLink);
+    dwError = TDNFSafeAllocateString(
+                  pRepoIn->pszMetaLink,
+                  &pRepo->pszMetaLink);
     BAIL_ON_TDNF_ERROR(dwError);
 
-    dwError = TDNFSafeAllocateString(pRepoIn->pszUrlGPGKey, &pRepo->pszUrlGPGKey);
+    dwError = TDNFSafeAllocateString(
+                  pRepoIn->pszUrlGPGKey,
+                  &pRepo->pszUrlGPGKey);
     BAIL_ON_TDNF_ERROR(dwError);
 
     *ppRepo = pRepo;
@@ -448,10 +526,6 @@ TDNFFreeReposInternal(
         TDNF_SAFE_FREE_MEMORY(pRepo->pszUrlGPGKey);
         TDNF_SAFE_FREE_MEMORY(pRepo->pszUser);
         TDNF_SAFE_FREE_MEMORY(pRepo->pszPass);
-        if(pRepo->hRepo)
-        {
-            hy_repo_free(pRepo->hRepo);
-        }
         pRepos = pRepo->pNext;
         TDNF_SAFE_FREE_MEMORY(pRepo);
     }
