@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 VMware, Inc. All Rights Reserved.
+ * Copyright (C) 2015-2017 VMware, Inc. All Rights Reserved.
  *
  * Licensed under the GNU Lesser General Public License v2.1 (the "License");
  * you may not use this file except in compliance with the License. The terms
@@ -26,7 +26,7 @@ TDNFRepoMakeCacheDirs(
     )
 {
     uint32_t dwError = 0;
-    if(!pszRepo)
+    if(IsNullOrEmptyString(pszRepo))
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
@@ -50,7 +50,7 @@ TDNFRepoGetBaseUrl(
     char* pszBaseUrl = NULL;
     PTDNF_REPO_DATA_INTERNAL pRepos = NULL;
 
-    if(!pTdnf || !pszRepo || !ppszBaseUrl)
+    if(!pTdnf || IsNullOrEmptyString(pszRepo) || !ppszBaseUrl)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
@@ -68,7 +68,7 @@ TDNFRepoGetBaseUrl(
         {
             break;
         }
-        pRepos = pRepos->pNext;  
+        pRepos = pRepos->pNext;
     }
 
     if(!pRepos)
@@ -79,7 +79,7 @@ TDNFRepoGetBaseUrl(
 
     dwError = TDNFAllocateString(pRepos->pszBaseUrl, &pszBaseUrl);
     BAIL_ON_TDNF_ERROR(dwError);
-    
+
     *ppszBaseUrl = pszBaseUrl;
 
 cleanup:
@@ -101,7 +101,7 @@ TDNFRepoGetUserPass(
     char* pszUserPass = NULL;
     PTDNF_REPO_DATA_INTERNAL pRepos = NULL;
 
-    if(!pTdnf || !pszRepo || !ppszUserPass)
+    if(!pTdnf || IsNullOrEmptyString(pszRepo) || !ppszUserPass)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
@@ -119,7 +119,7 @@ TDNFRepoGetUserPass(
         {
             break;
         }
-        pRepos = pRepos->pNext;  
+        pRepos = pRepos->pNext;
     }
 
     if(!pRepos)
@@ -128,7 +128,7 @@ TDNFRepoGetUserPass(
         BAIL_ON_TDNF_ERROR(dwError);
     }
 
-    if(!IsNullOrEmptyString(pRepos->pszUser) && 
+    if(!IsNullOrEmptyString(pRepos->pszUser) &&
        !IsNullOrEmptyString(pRepos->pszPass))
     {
         dwError = TDNFAllocateStringPrintf(
@@ -159,7 +159,7 @@ TDNFRepoGetRpmCacheDir(
     uint32_t dwError = 0;
     char* pszRpmCacheDir = NULL;
 
-    if(!pTdnf || !pszRepoId || !ppszRpmCacheDir)
+    if(!pTdnf || IsNullOrEmptyString(pszRepoId) || !ppszRpmCacheDir)
     {
         dwError = ERROR_TDNF_INVALID_PARAMETER;
         BAIL_ON_TDNF_ERROR(dwError);
@@ -276,6 +276,156 @@ error:
     goto cleanup;
 }
 
+uint32_t
+TDNFRemoveTmpRepodata(
+    const char* pszTmpRepodataDir,
+    const char* pszTmpRepoMDFile
+    )
+{
+    uint32_t dwError = 0;
+
+    if (IsNullOrEmptyString(pszTmpRepodataDir) || IsNullOrEmptyString(pszTmpRepoMDFile))
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+    if (unlink(pszTmpRepoMDFile))
+    {
+        if (errno != ENOENT)
+        {
+            dwError = errno;
+            BAIL_ON_TDNF_ERROR(dwError);
+        }
+    }
+    if (rmdir(pszTmpRepodataDir))
+    {
+        dwError = errno;
+        BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+    }
+cleanup:
+    return dwError;
+error:
+    goto cleanup;
+}
+
+uint32_t
+TDNFRemoveLastRefreshMarker(
+    PTDNF pTdnf,
+    const char* pszRepoId
+    )
+{
+    uint32_t dwError = 0;
+    char* pszLastRefreshMarker = NULL;
+
+    if(!pTdnf || !pTdnf->pConf || IsNullOrEmptyString(pszRepoId))
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    dwError = TDNFAllocateStringPrintf(
+                  &pszLastRefreshMarker,
+                  "%s/%s/%s",
+                  pTdnf->pConf->pszCacheDir,
+                  pszRepoId,
+                  TDNF_REPO_METADATA_MARKER);
+    BAIL_ON_TDNF_ERROR(dwError);
+    if (pszLastRefreshMarker)
+    {
+        if(unlink(pszLastRefreshMarker))
+        {
+           dwError = errno;
+           BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+        }
+    }
+cleanup:
+    TDNF_SAFE_FREE_MEMORY(pszLastRefreshMarker);
+    return dwError;
+error:
+    goto cleanup;
+}
+
+uint32_t
+TDNFRemoveSolvCache(
+    PTDNF pTdnf,
+    const char* pszRepoId
+    )
+{
+    uint32_t dwError = 0;
+    char* pszSolvCacheDir = NULL;
+    char* pszFilePath = NULL;
+    DIR *pDir = NULL;
+    struct dirent *pEnt = NULL;
+
+    if(!pTdnf || !pTdnf->pConf || IsNullOrEmptyString(pszRepoId))
+    {
+        dwError = ERROR_TDNF_INVALID_PARAMETER;
+        BAIL_ON_TDNF_ERROR(dwError);
+    }
+
+    dwError = TDNFAllocateStringPrintf(
+                  &pszSolvCacheDir,
+                  "%s/%s/%s",
+                  pTdnf->pConf->pszCacheDir,
+                  pszRepoId,
+                  TDNF_SOLVCACHE_DIR_NAME);
+    BAIL_ON_TDNF_ERROR(dwError);
+
+    pDir = opendir(pszSolvCacheDir);
+    if(pDir == NULL)
+    {
+        dwError = errno;
+        BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+    }
+
+    while ((pEnt = readdir (pDir)) != NULL )
+    {
+        if (!strcmp(pEnt->d_name, ".") || !strcmp(pEnt->d_name, ".."))
+        {
+            continue;
+        }
+
+        dwError = TDNFAllocateStringPrintf(
+                      &pszFilePath,
+                      "%s/%s",
+                      pszSolvCacheDir,
+                      pEnt->d_name);
+        BAIL_ON_TDNF_ERROR(dwError);
+        if(pszFilePath)
+        {
+            if(unlink(pszFilePath))
+            {
+                dwError = errno;
+                BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+            }
+            TDNF_SAFE_FREE_MEMORY(pszFilePath);
+            pszFilePath = NULL;
+        }
+        else
+        {
+            dwError = ERROR_TDNF_INVALID_PARAMETER;
+            BAIL_ON_TDNF_ERROR(dwError);
+        }
+    }
+    if(rmdir(pszSolvCacheDir))
+    {
+        dwError = errno;
+        BAIL_ON_TDNF_SYSTEM_ERROR(dwError);
+    }
+
+cleanup:
+    TDNF_SAFE_FREE_MEMORY(pszFilePath);
+    TDNF_SAFE_FREE_MEMORY(pszSolvCacheDir);
+    if(pDir)
+    {
+        closedir(pDir);
+    }
+    return dwError;
+
+error:
+    goto cleanup;
+}
+
 uint32_t
 TDNFRepoApplyProxySettings(
     PTDNF_CONF pConf,
@@ -293,9 +443,9 @@ TDNFRepoApplyProxySettings(
     if(!IsNullOrEmptyString(pConf->pszProxy))
     {
         if(curl_easy_setopt(
-            pCurl,
-            CURLOPT_PROXY,
-            pConf->pszProxy) != CURLE_OK)
+               pCurl,
+               CURLOPT_PROXY,
+               pConf->pszProxy) != CURLE_OK)
         {
             dwError = ERROR_TDNF_SET_PROXY;
             BAIL_ON_TDNF_ERROR(dwError);
@@ -304,9 +454,9 @@ TDNFRepoApplyProxySettings(
         if(!IsNullOrEmptyString(pConf->pszProxyUserPass))
         {
             if(curl_easy_setopt(
-                pCurl,
-                CURLOPT_PROXYUSERPWD,
-                pConf->pszProxyUserPass) != CURLE_OK)
+                   pCurl,
+                   CURLOPT_PROXYUSERPWD,
+                   pConf->pszProxyUserPass) != CURLE_OK)
             {
                 dwError = ERROR_TDNF_SET_PROXY_USERPASS;
                 BAIL_ON_TDNF_ERROR(dwError);
